/* mp3 live server version 0.02 */
/* Copyright Scott Manley 1997 */
/* This Software is protected under the terms of the Gnu Public License */
/* All code is original*/

/* history                                                                  */
/* v0.01 - OK decided that I need to Give things version No.s               */
/*         Made the reflector clever enough to strip of garbage before      */
/*                  headers from a reflected stream                         */
/*         Reflector Strips of >1024 bytes from head of stream to make live */
/*                  headers consistent                                      */
/*         The header.mp3 file is not used any more but is still generated  */
/*                  I can't rememebr why ;-) but I had a reason earlier     */
/*                  (I now rememebr it's for using multiple live files      */
/*                   To save on disk space)                                 */
/*         Standardised the error Reporting system                          */
/*         Add checks to see if live.mp3 is increasing with time            */

/* v0.02   Did the decent thing and split the code into lots of files       */
/*         Created proper command line configuration options                */
/*         Created extra maintenance process for handling all the logs      */
/*                  open connections and other stuff                        */

/* v0.03   The Program can now spawn l3enc using the -s option              */
/*         It can accept input directly from teh soudcard for serving       */
/*         -n limit option works by keeping a count of subprocesses         */

/* v0.04   Reflector Now Works again!!!!! now I'm almost back to where I was*/
/*         now all I need ot do is fix the stream reconnect code - phew     */

/* main server code this does most of the inital setup before spawning  */ 
/* specialised processes                                                */

#include "mp3serv.h"
extern int errno;

/* default names */ 
char maintenance_pipe[]="pipe.mnt";
char sound_pipe[]="sound.raw";
char header_file[]="header.mp3";
char live_file[]="live.mp3";
char log_file[]="mp3serv.log";




/* Fatal error Handler */
void fatal(char *text)
{
    fprintf(stderr, "%6d:Error: %s\n",getpid(),text);
    exit(1);
}

void do_config(int argc,char *argv[],config *conf){
int i;
/*defaults */
(*conf).serv_port=PORT;
(*conf).mnt_port=PORT+1;
(*conf).server==-1;
(*conf).reflector_port=PORT;
(*conf).mnt_pipe=maintenance_pipe;
(*conf).snd_pipe=sound_pipe;
(*conf).header_file=header_file;
(*conf).live_file=live_file;
(*conf).log_file=log_file;
(*conf).limit=10;
for(i=1;i<argc;i++){
  if(argv[i][0]=='-'){
    switch(argv[i][1]){
    case 'p':
    case 'P':
      i++;
      (*conf).serv_port=atoi(argv[i]);
      break;
    case 's':
    case 'S':
      (*conf).server=1;
      break;
    case 'm':
    case 'M':
      i++;
      (*conf).mnt_port=atoi(argv[i]);
      break;
    case 'r':
    case 'R':
      i++;
      (*conf).server=0;
      (*conf).reflector=argv[i];
      break;
    case 'e':
    case 'E':
      i++;
      (*conf).reflector_port=atoi(argv[i]);
      break;
    case 'c':
    case 'C':
      i++;
      (*conf).mnt_pipe=argv[i];
      break;
    case 'x':
    case 'X':
      (*conf).server=-1;
      break;
    case 'n':
    case 'N':
      i++;
      (*conf).limit=atoi(argv[i]);
      break;
    default:
      fatal("Unrecognised command line switch");
      break;
    } 
  } else {
    fatal("Unrecognised command line argument");
  }
}
}


/* General sort of paranoia check to ensure that we can do the thigns we */
/* want to with the files we've specified */
void setup_pipes(config conf){
struct stat stats;
FILE *test;
if(mkfifo(conf.mnt_pipe,S_IRUSR | S_IWUSR)) {
  stat(conf.mnt_pipe,&stats);
  if(!(stats.st_mode|S_IFIFO))
    fatal("maintenance pipe file exists and is not a pipe");
  if(!(stats.st_mode|(S_IWUSR|S_IRUSR)))
    fatal("cannot read/write maintenance pipe");
  }
if(conf.server!=-1){
  if((test=fopen(conf.live_file,"wb"))==NULL)
    fatal("cannot open live file for writing");
  fclose(test);
}
if((test=fopen(conf.log_file,"wb"))==NULL)
  fatal("cannot open live file for writing");
fclose(test);
if((test=fopen(conf.header_file,"wb"))==NULL)
  fatal("cannot open live file for writing");
fclose(test);
if((conf.server)&&(conf.server!=-1)){
  if(mkfifo(conf.snd_pipe,S_IRUSR | S_IWUSR)) {
    stat(conf.snd_pipe,&stats);
    if(!(stats.st_mode|S_IFIFO))
      fatal("sound pipe file exists and is not a pipe");
    if(!(stats.st_mode|(S_IWUSR|S_IRUSR)))
      fatal("cannot read/write sound pipe");
  }
}
}

void shutdown_server(config conf){
  /* this will do something soon */
}

/* new main loop */
void main(int argc,char *argv[]){
config conf;

do_config(argc,argv,&conf);
setup_pipes(conf);

if(conf.server!=-1){
  switch (fork())
    {
    case 0:
      if(conf.server)
	start_recording(conf);
      else
	get_server_feed(conf);
      
    case -1:
      fatal("fork() - sound process");
    }
}
switch (fork())
      {
      case 0:
      start_server(conf);
      
      case -1:
	fatal("fork() - server Process");
      }

maintain_server(conf);
shutdown_server(conf);
exit(0);
}
